import logging
import unittest
from typing import Any, Dict

from dynatrace_extension import Extension
from dynatrace_extension.sdk.runtime import RuntimeProperties


class TestRuntimeProperties(unittest.TestCase):
    def tearDown(self) -> None:
        Extension._instance = None

    def test_api_log_level(self) -> None:
        response_json: Dict[str, Any] = {"runtime": {}}
        response_json["runtime"]["debuglevel.extension1.api"] = "debug"  # converted to debug
        response_json["runtime"]["debuglevel.extension2.api"] = "WARNING"  # wrong value - default to info

        runtime = RuntimeProperties(response_json)

        self.assertEqual(logging.DEBUG, runtime.get_api_log_level("extension1"))
        self.assertEqual(logging.INFO, runtime.get_api_log_level("extension2"))
        self.assertEqual(logging.INFO, runtime.get_api_log_level("extension3"))



import unittest
from unittest.mock import MagicMock, mock_open, patch

from dynatrace_extension.sdk.communication import HttpClient, divide_into_chunks


class TestCommunication(unittest.TestCase):
    def test_large_chunk(self):
        large_list = ["metric 1"] * 1400
        chunks = list(divide_into_chunks(large_list, 1000))
        self.assertEqual(len(chunks), 2)
        self.assertEqual(len(chunks[0]), 1000)
        self.assertEqual(len(chunks[1]), 400)

    def test_small_chunk(self):
        small_list = ["metric 1"] * 10
        chunks = list(divide_into_chunks(small_list, 1000))
        self.assertEqual(len(chunks), 1)
        self.assertEqual(len(chunks[0]), 10)

    @patch("builtins.open", mock_open(read_data="test_token"))
    @patch.object(HttpClient, "_make_request", return_value=MagicMock())
    def test_http_client_metric_report(self, mock_make_request):
        http_client = HttpClient("https://localhost:9999", "1", "token", MagicMock())
        few_metrics = ["metric 1", "metric 2"]
        responses = http_client.send_metrics(few_metrics)
        self.assertEqual(len(responses), 1)

        many_metrics = ["metric 1"] * 1400
        responses = http_client.send_metrics(many_metrics)
        self.assertEqual(len(responses), 2)

        no_metrics = []
        responses = http_client.send_metrics(no_metrics)
        self.assertEqual(len(responses), 0)



import unittest
from datetime import datetime

from dynatrace_extension import Metric, MetricType
from dynatrace_extension.sdk.metric import SfmMetric


class TestMetric(unittest.TestCase):
    def test_simple(self):
        metric = Metric("myMetric", 101)
        self.assertEqual(metric.to_mint_line(), "myMetric gauge,101")

    def test_delta(self):
        metric = Metric("myMetric", 101, metric_type=MetricType.DELTA)
        self.assertEqual(metric.to_mint_line(), "myMetric count,delta=101")

    def test_dimensions(self):
        metric = Metric("myMetric", 101, dimensions={"someid": "42", "descr": "Interface73/0/10"})
        self.assertEqual(metric.to_mint_line(), 'myMetric,someid="42",descr="Interface73/0/10" gauge,101')

    def test_timestamp(self):
        timestamp = datetime(2022, 1, 1, 1, 0, 0, 0)
        metric = Metric("myMetric", 101, timestamp=timestamp)
        self.assertEqual(metric.to_mint_line(), f"myMetric gauge,101 {int(timestamp.timestamp() * 1000)}")

    def test_too_many_dimensions(self):
        dimensions = {k: "value" for k in range(0, 51)}
        metric = Metric("myMetric", 101, dimensions=dimensions)
        self.assertRaises(ValueError, metric.validate)

    def test_line_too_large(self):
        metric = Metric("a" * 2000, 101)
        self.assertRaises(ValueError, metric.validate)


class TestSfmMetric(unittest.TestCase):
    def test_sfm_metric_key(self):
        metric = SfmMetric("callback.execution.time", 1, client_facing=True)
        self.assertEqual(metric.key, "dsfm:datasource.python.callback.execution.time")

        metric = SfmMetric("callback.execution.time", 1)
        self.assertEqual(metric.key, "isfm:datasource.python.callback.execution.time")
